/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.request;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import kieker.webgui.domain.User.Role;

/**
 * This simple bean is request scoped and can be used to store the necessary data for a new user during a request. It is not to be used to deliver a list of users
 * for example, as there is a field for a password.
 * 
 * @author Nils Christian Ehmke
 */
@Component
@Scope("request")
public class NewUserBean {

	private Role role;
	private String username;
	private String password;
	private boolean active;

	/**
	 * Creates a new instance of this bean and initializes it with empty fields. <b>Do not use this constructor. This bean is Spring managed.</b>
	 */
	public NewUserBean() {
		this.role = Role.ROLE_GUEST;
		this.username = "";
		this.password = "";
		this.active = false;
	}

	/**
	 * Getter for the property {@link NewUserBean#role}.
	 * 
	 * @return The current value of the property.
	 */
	public Role getRole() {
		return this.role;
	}

	/**
	 * Setter for the property {@link NewUserBean#role}.
	 * 
	 * @param role
	 *            The new value for the property.
	 */
	public void setRole(final Role role) {
		this.role = role;
	}

	/**
	 * Delivers the current value of the property {@link NewUserBean#username}.
	 * 
	 * @return The current value of the property.
	 */
	public String getUsername() {
		return this.username;
	}

	/**
	 * Setter for the property {@link NewUserBean#username}.
	 * 
	 * @param username
	 *            The new value for the property.
	 */
	public void setUsername(final String username) {
		this.username = username;
	}

	/**
	 * Delivers the current value of the property {@link NewUserBean#password}.
	 * 
	 * @return The current value of the property.
	 */
	public String getPassword() {
		return this.password;
	}

	/**
	 * Setter for the property {@link NewUserBean#password}.
	 * 
	 * @param password
	 *            The new value for the property.
	 */
	public void setPassword(final String password) {
		this.password = password;
	}

	/**
	 * Delivers the current value of the property {@link NewUserBean#isActive}.
	 * 
	 * @return The current value of the property.
	 */
	public boolean isActive() {
		return this.active;
	}

	/**
	 * Setter for the property {@link NewUserBean#isActive}.
	 * 
	 * @param active
	 *            The new value for the property.
	 */
	public void setActive(final boolean active) {
		this.active = active;
	}

}
