/**************************************************************************

Copyright [2012] [Software Engineering Group - Kiel University]

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

***************************************************************************/

var labelType, useGradients, nativeTextSupport, animate;

/**
	Set up Error Text and Navigator
*/

(function() {
  var ua = navigator.userAgent,
      iStuff = ua.match(/iPhone/i) || ua.match(/iPad/i),
      typeOfCanvas = typeof HTMLCanvasElement,
      nativeCanvasSupport = (typeOfCanvas == 'object' || typeOfCanvas == 'function'),
      textSupport = nativeCanvasSupport 
        && (typeof document.createElement('canvas').getContext('2d').fillText == 'function');
  //I'm setting this based on the fact that ExCanvas provides text support for IE
  //and that as of today iPhone/iPad current text support is lame
  labelType = (!nativeCanvasSupport || (textSupport && !iStuff))? 'Native' : 'HTML';
  nativeTextSupport = labelType == 'Native';
  useGradients = nativeCanvasSupport;
  animate = !(iStuff || !nativeCanvasSupport);
})();

var Log = {
  elem: false,
  write: function(text){
    if (!this.elem) 
      this.elem = document.getElementById('log');
    this.elem.innerHTML = text;
    this.elem.style.left = (500 - this.elem.offsetWidth / 2) + 'px';
  }
};

		
function GraphFlow(){
	
	/////////////////////////////////////////////
	//				VARIABLES				   //
	/////////////////////////////////////////////
	
	var visContainer = 'infovis';
	
	/** Colors */
	var nodeFillColor = [];
		nodeFillColor['nodeFamily']		= "#DEDEDE";
	
	var nodeStrokeColor = [];
		nodeStrokeColor['inputPort']   	  = "#AA0000";
		nodeStrokeColor['outputPort']	  = "#AA0000";
		nodeStrokeColor['repositoryPort'] = "#AA0000";
		nodeStrokeColor['nodeFamily'] 	  = "#4D4D4D";
		nodeStrokeColor['crossBox'] 	  = "#4D4D4D";
		
	var nodeColorFocus	  = "#0098BE",
		edgeColor 		  = "#114270",
		edgeColorFocus 	  = "#0098BE";
		
	/** Icon Images */
	var nodeIcons = [];
		
	/** Overall size modifier: the width of node labels in pixels */
	var vertexLabelSize 	= 12;

	/** FlowGraph Object */
	var fd;

	/** Nodes and edges that are highlighted when the mouse moves over them */
	var hover = null;
	var markedNode = null;

	/** This array stores all Nodes and is initialized with a dummy,
		used for drawing edges between a node and the mousepointer. Empty
		elements are dynamically reserved for additional nodes.
		*/
	var json;
		
	/** Used for dynamically adding and removing space for node creation/deletion
		resulting in better performance when manipulating the json array.
	*/
	var jsonCapacity = {'occupied' : 0,
						'max'  : 10};

	/** selected node:	{id : the id of the selected node, 
						 from : the data.$type of the selected node,
						 label: the label of a selected edge}
						 
		When clicking a node or edge, this variable will contain node
		information, that is required for attaching/detaching edges.
		*/
	var selectedNode = null;
	
	/** This points to a DOM-node (not the json-array node!) which is
		an invisible dummy for moving an edge with the mouse
		*/
	var mouseNode = null;

	/** true if the mouse is over a port/edge */
	var mouseOverNode = false;
	var mouseOverEdge = false;
	
	/** A hashtable, mapping mouse and node events to arrays of functions that
		are called when these events happen
		*/
	var listener = [];
	
	/** Can be changed to allow or prohibit node movement or deletion/creation */
	var readOnly = {'deleteCreate' 	: true,
					'move' 			: true};
	
	/** Grid dummy-node which is drawn over the graph if it is visible */
		
	var grid = { 'data': {'$dim' : vertexLabelSize*4,
						  '$type'	 : 'grid',
						  '$color'  : '#AAAAFF',
						  '$width' : document.getElementById(visContainer).clientWidth,
						  '$height': document.getElementById(visContainer).clientHeight
						},
				 'id':	'#DUMMY_GRID_NODE',
				 'visible': false,
				 'snap'   : false
				};
	
	/** Stores information about the canvas, such as its dimensions and its position.
		This information is needed to correctly draw the Grid and may come in handy in
		future functions.
		*/
	var navi = { 'width' : document.getElementById(visContainer).clientWidth,
				 'height': document.getElementById(visContainer).clientHeight,
				 'centerX' : 0,
				 'centerY' : 0,
				 'dragX' : null,
				 'dragY' : null,
				 'draggedNodes' : null};
	
	/////////////////////////////////////////////
	//				FUNCTIONS				   //
	/////////////////////////////////////////////

	/**
		Concatenates nodefamily 'id's and their x- and y-position
		to a string, separating each element with a space.
		TODO: save bend points of edges
		@return - the resulting string
	*/
	this.saveNodePositions = function(){
		var positions = "";
		var node;
		
		for(var n = 0, l = jsonCapacity.occupied; n < l; n++){
			node = json[n];
			positions += node.id + " " + node.data.$xPos + " " + node.data.$yPos + " ";
		}
		
		return positions;
	}
	
	/**
		Reads nodefamily positions out of a string, which was created with
		the saveNodePositions() function and moves all nodes to the respective
		positions.
		@param positions - the string which contains node ids and positions
		@param smoothTransition - no functionality at the moment
	*/
	this.loadNodePositions = function(positions, smoothTransition){
		var pos = positions.split(" ");
		var node, x, y;
		
		for(var i = 0, l = pos.length; i < l; i+=3){
			// look for the node
			node = fd.graph.getNode(pos[i]);
			
			// if node exists, set positions
			if(node){
				x = parseFloat(pos[i+1]);
				y = parseFloat(pos[i+2]);
				
				prepareNodeMove(node);
				moveNode(x,y);
				saveNodeMove();
			}
		}
		
		//TODO: smooth animation
		if(false){
			fd.animate({  
				modes: ['linear'],
				transition: $jit.Trans.Cubic.easeInOut,
				duration: 1000
			}); 
		}
		else{
			fd.plot();
		}
	}
	
	/**
		Returns a XML-String, representing the graph as KGraph
		This function is work in progress and will be tuned
		for efficiency in the future!
	*/
	this.convertToKGraph = function(){
		// define helper strings
		var ie = '<children incomingEdges="';
		var pt = '<ports edges="';
		var oe = [	'<outgoingEdges target="//@children.',
								'" sourcePort="//@children.',
								'/@ports.',
								'" targetPort="//@children.',
								'">\n<data xsi:type="klayoutdata:KEdgeLayout"/>\n</outgoingEdges>\n'					
				 ];
		
		// this array will contain XMI-Strings
		var children = [],
			indexMap = [];		// KGraphs use indices instead of IDs
		
		// this loop prepares the children array
		var child;
		for(var n = 0, l = jsonCapacity.occupied; n < l; n++){
			// reserve space for edges
			child = {
						incomingEdges : ie,
						ports :	[],
						outgoingEdges : ""
					};
			// reserve space for port strings
			for(var p = 0, pl = json[n].data.$portCount-1; p < pl; p++){ // TODO: adjust for ReadOnly
					child.ports.push({"text":pt, "isEast":false});
			}
			children.push(child);
			
			// add node id to index map
			indexMap[json[n].id] = n;
		}
		
		// determine port indices
		var node,
			indexCounter = 0;
		for(var n = jsonCapacity.max, l = json.length; n < l; n++){
			node = json[n];
			if(node.data.$type == "crossBox"){
				indexCounter = 0;
			}
			else{
				indexMap[json[n].id] = indexCounter;
				indexCounter++;
			}
		}
		
		// now prepare all strings needed
		indexCounter = -1;
		var edgeCounter = 0, portCounter = 0;
		var adja, edge, edgeString, targetChild, targetPort, relativeY;
		for(var n = jsonCapacity.max+1, l = json.length; n < l; n++){
			node = json[n];
			if(node.data.$type == "crossBox"){ // TODO: do not rely on crossBox, make more generic
				indexCounter++;
				edgeCounter = 0;
				portCounter = 0;
			}
			else{
				// is the port the first of its type within the node (e.g. the first inputPort)
				relativeY = node.data.$relativeY;
				if(relativeY){
					children[indexCounter].ports[portCounter].yPos = relativeY;
				}
			
				// is the port positioned on the east side of the node?
				children[indexCounter].ports[portCounter].isEast = node.data.$type == "inputPort";
			
				adja = node.adjacencies;
				for(var e = 0, el = adja.length; e < el; e++){
					edge = adja[e];
					edgeString = "//@children."
								+ indexCounter 
								+"/@outgoingEdges." 
								+ edgeCounter + " ";
					targetChild = indexMap[edge.data.$targetFamily];
					targetPort = indexMap[edge.nodeTo];
					
					// add edge info to outgoing family
					children[indexCounter].outgoingEdges +=
						oe[0] + targetChild + oe[1] + indexCounter + oe[2] + portCounter 
						+ oe[3] + targetChild + oe[2] + targetPort + oe[4];
					children[indexCounter].ports[portCounter].text += edgeString; 
					
					// add edge info to target family
					children[targetChild].ports[targetPort].text += edgeString;
					children[targetChild].incomingEdges += edgeString;
					
					edgeCounter++;
				}
				
				portCounter++;
			}
		}
		// redefine helper strings
		ie = '">\n<data xsi:type="klayoutdata:KShapeLayout" xpos="0.0" ypos="0.0" width="';
		pt = '">\n<data xsi:type="klayoutdata:KShapeLayout" xpos="';
		var ie2 = '">\n<persistentEntries key="de.cau.cs.kieler.sizeConstraint" value="FIXED"/>\n'
					+ '<persistentEntries key="de.cau.cs.kieler.portConstraints" value="FIXED_POS"/>\n<insets/>\n</data>\n',
			pt2 = '" width="'+(vertexLabelSize*2)+'" height="'+vertexLabelSize+'">\n</data>\n</ports>\n';
		
		// assemble XMI-String
		var xmi = '<?xml version="1.0" encoding="UTF-8"?>\n'
					+ '<kgraph:KNode xmi:version="2.0" xmlns:xmi="http://www.omg.org/XMI" '
					+ 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:kgraph="http://kieler.cs.cau.de/KGraph" '
					+ 'xmlns:klayoutdata="http://kieler.cs.cau.de/KLayoutData">\n<data xsi:type="klayoutdata:KShapeLayout" '
					+ 'width="348.0" height="112.0">\n<persistentEntries key="de.cau.cs.kieler.sizeConstraint" value="FIXED"/>\n'
					+ '<insets/>\n</data>\n\n';
		var ports, xPos, yPos, width, height;
		for(var c = 0, l = children.length; c < l; c++){
			child = children[c];
			width = json[c].data.$width;
			height = json[c].data.$height;
			xmi += child.incomingEdges + ie + width + '" height="'+ height + ie2;
			
			ports = child.ports;
			for(var p = 0, pl = ports.length; p < pl; p++){
			
				// determine the y-Position of the port
				relativeY = ports[p].yPos;
				if(relativeY){
					yPos = relativeY + height/2;
				}
				else{
					yPos += 2*vertexLabelSize;
				}
				
				// determine the x-Position of the port
				if(ports[p].isEast){
					xPos = -vertexLabelSize;
				}
				else {
					xPos = width - vertexLabelSize;
				}
				xmi += ports[p].text + pt + xPos + '" ypos="'+ yPos +pt2;
			}
			
			xmi += child.outgoingEdges + '</children>\n';
			
			delete children[c];
		}
		xmi += '</kgraph:KNode>';
		
		return xmi;
	}
	
	/**
		Checks an input color-string for validation.
		Sends an onError-event if the string is not valid.
		@param callFunction - the function where the argument needs to be checked
		@param color - the color argument which is checked
		@returns - false if the string is not a well formed color string 
	*/
	function validArgColor(callFunction, color){
		var valid = false;
		
		// is it a string at all ?
		if(typeof color == "string"){
			// does the string match "#RRGGBB" ?
			var regEx = /^#[a-f\d]{6}$/i;
			valid = color.match(regEx);
		}
		
		if(!valid){
			callListener("onError", ["Error in function "+ callFunction 
										+ ": Invalid color '"+ color 
										+ "'. Must be structured like '#RRGGBB'."]);
		}
		return valid;
	}
	
	
	/**
		Checks an input object for validation.
		Sends an onError-event if the Object does not have an id
		@param callFunction - the function where the argument needs to be checked
		@param node - the object argument which is checked
		@returns - false if the object does not have an id
	*/
	function validArgNode(callFunction, node){
		var valid = false;
		
		// is it an object at all ?
		if(node && typeof node == "object"){
			// does the object have an id
			valid = (typeof node.id != "undefined");
		}
		
		if(!valid){
			callListener("onError", ["Error in function "+ callFunction 
										+ ": Invalid Node '"+ node 
										+ "'. Must have a field 'id'."]);
		}
		return valid;
	}
	
	
	/**
		Checks an argument for type correctness.
		Sends an onError-event if the argument is not valid.
		@param callFunction - the function where the argument needs to be checked
		@param arg - the argument which is checked
		@param type - the type of which the argument must be
		@returns - false if the string is not a well formed color string 
	*/
	function validArg(callFunction, arg, type){
		var valid = (typeof arg == type);
		if(!valid){
			callListener("onError", ["Error in function "+ callFunction 
										+ ": Invalid argument type for argument '"
										+ arg + "'. Must be of type " + type + "."]);
		}
		return valid;
	}
	
	/**
		Changes the color of the grid. Requires a refresh().
	*/
	this.setGridColor = function(newColor){
		if(validArgColor('setGridColor()', newColor)){
			grid.data.$color = newColor;
		}
	}
	
	/**
		Changes the size of the grid. Requires a refresh().
	*/
	this.setGridSize = function(newSize){
		if(validArg('setGridSize()', newSize, 'number')){
			grid.data.$dim = newSize;
		}
	}
	
	/**
		Changes the visibility of the grid. Requires a refresh().
	*/
	this.setGridVisible = function(visibility){
		grid.visible = visibility;
	}
	
	/**
		Changes whether Nodes should snap to grid. Requires a refresh().
	*/
	this.setGridSnap = function(snap){
		grid.snap = snap;
	}
	
	/**
		Scales and centers the graph to fit the space provided by the container
		in which it is drawn.
	*/
	this.scaleToFit = function(){
	
	// do nothing if the graph is empty
	if(jsonCapacity.occupied == 0){
		return;
	}
	
	// init loop variables
	var data   = json[0].data,
		width  = data.$width/2,
		height = data.$height/2;
	
	var left  	= data.$xPos - width,
		right 	= data.$xPos + width, 
		top		= data.$yPos - height, 
		bottom	= data.$yPos + height;
	
	// init the outer bounds of the displayed graph
	var leftMost = left,
		rightMost = right,
		topMost = top,
		bottomMost = bottom;
	
	// calculate the outer bounds of the displayed graph
	for(var n = 1, l= jsonCapacity.occupied; n < l; n++){
		data   = json[n].data,
		width  = data.$width/2,
		height = data.$height/2;
	
		left  	= data.$xPos - width,
		right 	= data.$xPos + width, 
		top		= data.$yPos - height, 
		bottom	= data.$yPos + height;
		
		// update bounds
		if(left < leftMost){
			leftMost = left;
		}
		if(right > rightMost){
			rightMost = right;
		}
		if(top < topMost){
			topMost = top;
		}
		if(bottom > bottomMost){
			bottomMost = bottom;
		}
	}
	
	// scaling values
	var oldScale = fd.canvas.scaleOffsetX,
		scaleX = navi.width / (rightMost - leftMost + 4*vertexLabelSize),
		scaleY = navi.height / (bottomMost - topMost +4*vertexLabelSize)
		scale = Math.min(scaleX, scaleY) / oldScale;
		
	// translation values
	var midX = (leftMost + rightMost) / 2,
		midY = (topMost + bottomMost) / 2 -vertexLabelSize;
	
	// translate to origin, scale, and translate to the center of the graph
	fd.canvas.translate( -fd.canvas.translateOffsetX/oldScale, -fd.canvas.translateOffsetY/oldScale);
	fd.canvas.scale(scale, scale);
	fd.canvas.translate( -midX, -midY);
	}
	
	
	/**
	Assigns an iconPath to a type of node. Needs a refresh() to take effect.
	@param nodeType - the type of the node. Typically 'Repository',
					  'Filter', or 'Reader'
	@param path - the path to the image. If null, an existing
				  icon will be deleted
	*/
	this.setNodeIcon = function(nodeType, path){
		if(!validArg('setNodeIcon()', nodeType, 'string')){
			return;
		}
	
		if(path == null){
			delete nodeIcons[nodeType];
			//TODO: adjust shrink width
		}
		else{
			if(!validArg('setNodeIcon()', path, 'string')){
				return;
			}
			var icon = Image();
			icon.src = path;
			
			// if the image was loaded successfully, its width must be greater 0
			if(icon.width > 0){
				// if nodes were already created, we need to
				// adjust their width and iconPath
				if(nodeIcons[nodeType] == undefined){
					var	addSize = 6* vertexLabelSize,
						addPortSize = addSize / 2,
						max = jsonCapacity.max;
				
					var data, portData, portCount, addPortSizeRel;
					for(var n = 0, l = jsonCapacity.occupied; n < l; n++){
						// adjust node width
						data = json[n].data;
						if(data.$nodeType == nodeType){
							data.$icon = icon;
							data.$width += addSize;
							portCount = data.$portCount;
							
							// adjust port positions
							for(var p = data.$portIndex+max, s = p+portCount; p < s; p++){
								portData = json[p].data;
								if(portData.$type == "inputPort"){
									addPortSizeRel = -addPortSize;
								}else{
									addPortSizeRel = addPortSize;
								}
								if(portData.$relativeX != undefined){
									portData.$relativeX += addPortSizeRel;
								}
								portData.$xPos += addPortSizeRel;
							}
						}
					}
				}
				nodeIcons[nodeType] = icon;
			}
			else{
				callListener("onError", ["Error in function setNodeIcon(): Could not load '"+ path +"'."]);
			}
		}
	}
	
	/**
	Changes the appearance of the mouse cursor. 
	@param newCursor - 	a String-name of the new cursor. 
						If null, restore the cursor.
	*/
	this.setMouseCursor = function(newCursor){
		if(newCursor == null){
			fd.canvas.getElement().style.cursor = '';
			
		}else if(validArg('setMouseCursor()', newCursor, 'string')){
			fd.canvas.getElement().style.cursor = newCursor;
		}
	}
	
	
	/** 
		Reloads the graph, restoring all positions and displaying all
		changes to the graph that happened since the last refresh.
		This function needs to be called whenever nodes or egdges are
		added or deleted!
	*/
	this.refresh = function(){
		// reload graph
		var mouseIndex = jsonCapacity.max;
		
		fd.loadGraphFlowJSON(json, mouseIndex);
		
		if(grid.visible){
			fd.graph.addNode(grid);
		}
		fd.plot();
		
		// set the mouseNode to the freshly created graph node
		mouseNode = fd.graph.getNode(json[mouseIndex].id);
	}
	
	/**
		Adds a functionality to the following MouseEvents:
			onClick,
			onRightClick,
			onMouseMove,
			onMouseEnter,
			onMouseLeave,
			onDragStart,
			onDragMove,
			onDragCancel,
			onDragEnd
		Or one of these events:
			onError(errorMessage)
			onCreateEdge(sourceNode, targetNode, sourcePort, targetPort) : boolean,
			onCreateNode(node) : boolean,
			onRemoveEdge(sourceNode, targetNode, sourcePort, targetPort) : boolean,
			onRemoveNode(node) : boolean
		@param eventName - the name of the mouse event ( see list above)
		@param listenerFunction - a function that may alter the graph
	*/
	this.addListener = function(eventName, listenerFunction){
		// check arguments
		if(!validArg('addListener()', eventName, 'string') ||
			!validArg('addListener()', listenerFunction, 'function')){
				return;
		}
	
		if(listener[eventName] == undefined){
			listener[eventName] = [];
		}
		listener[eventName].push(listenerFunction);
	}
	
	/**
		Calls all listeners that are registered under the eventName.
		@param eventName - see 'addListener()'
		@param arguments - an array of arguments. For mouseEvents it is [node, eventInfo, e],
						   for nodeEvents it is [node], 
						   and for edgeEvents it is [sourceNode, targetNode, sourcePort, targetPort]
		@return true if an editor action is permitted
	*/
	function callListener(eventName, arguments){
		var lArr = listener[eventName];
		
		if(!lArr){
			return true;
		}
		var permitted = true;
		var testval;
		
		// iterate through all listener this event concerns
		
		// mouse listener functions with 3 arguments (node, eventInfo, e)
		if(arguments.length == 3){
			for(var l=0; l < lArr.length; l++){
				lArr[l](arguments[0], arguments[1], arguments[2]);
			}
			
		// node listener functions with 1 argument (node)
		}else if(arguments.length == 1){
			for(var l=0; l < lArr.length; l++){
				testval = lArr[l](arguments[0]);
				if(testval == false){
					permitted = false;
				}
			}
		
		// edge listener functions with 4 arguments (sourceNode, targetNode, sourcePort, targetPort)
		}else if(arguments.length == 4){
			for(var l=0; l < lArr.length; l++){
				testval = lArr[l](arguments[0], arguments[1], arguments[2], arguments[3]);
				if(testval == false){
					permitted = false;
				}
			}
		}
		return permitted;
	}
	
	/**
	Adds a listener that prohibits edge creation if...
		... an edge with the same source and target exists.
		...	the target node is the source node.
		... the source and target node are of the same port type.
		... a repository node is not connected to a repository and
			vice versa.
	*/
	this.addEdgeConstraints = function(){
		addListener("onCreateEdge", 
		function(sourceFamily, targetFamily, sourcePort, targetPort){
		
			// remove Edge if it does not lead to an inputPort
			if(targetPort.data.$type != "inputPort"){
				return false;
			}
		
			// remove Edge if the connected nodes share the same type
			if(sourcePort.data.$type == targetPort.data.$type){
				return false;
			}
			
			var goesToRepo = (targetFamily.data.$nodeType == "Repository"),
				isRepoPort = (sourcePort.data.$type == "repositoryPort");
			
			// remove Edge if it is falsely connected to repository ports
			if(goesToRepo != isRepoPort){
				return false;
			}
			
			// remove duplicate Edge
			var adja = sourcePort.adjacencies,
				duplicate = false;
			for(var a=0; a < adja.length; a++){
				if(adja[a].nodeTo == targetPort.id){
					if(duplicate){
						return false;
					}
					else{
						duplicate = true;
					}
				}
			}
			return true;
		});
	}
	
	/**
	Iterates through all nodes and ports, calling a given function for each one.
	@param nodeFunction - a single parameter function, called on a node.
	*/
	this.iterateAllNodes = function(nodeFunction){
		// check arguments
		if(validArg('iterateAllNodes()', nodeFunction, 'function')){
			return;
		}
	
		var node;
		for(var n=0, l=jsonCapacity.occupied; n < l; n++){
			node = json[n];
			nodeFunction(node);
		}
		for(var n=jsonCapacity.max+1, l= json.length; n < l; n++){
			node = json[n];
			nodeFunction(node);
		}
	}
	
	/**
	Changes the colors of the entire graph.
	Requires a refresh().
	@param fillColor - the fill color of nodes. If null, restore Node Color from global Array.
	@param strokeColor - the stroke color of nodes. If null, restore Stroke Color from global Array.
	@param portColor - the color of all ports. If null, restore Port Color from global Array.
	@param edgeColor - the color of edges. If null, restore Edge Color from global Array.
	@param highlightColor - the of highlighted objects. If null, restore Edge Color from global Array.
	*/
	this.setNodeStyle = function(fillColor, strokeColor, portColor, edColor, fcColor){
		
		if(fillColor && validArgColor('setNodeStyle()', fillColor)){
			nodeFillColor['nodeFamily'] = fillColor;
		}
		if(strokeColor && validArgColor('setNodeStyle()', strokeColor)){
			nodeStrokeColor['nodeFamily'] = strokeColor;
		}
		
		if(portColor && validArgColor('setNodeStyle()', portColor)){
			nodeStrokeColor['inputPort'] = portColor;
			nodeStrokeColor['outputPort'] = portColor;
			nodeStrokeColor['repositoryPort'] = portColor;
		}
		if(edColor && validArgColor('setNodeStyle()', edColor)){
			edgeColor = edColor;
		}
		if(fcColor && validArgColor('setNodeStyle()', fcColor)){
			edgeColorFocus = fcColor;
			nodeColorFocus = fcColor;
		}
		
		
		var data, a, al, adja;
		// change family colors
		for(var n = 0, l = jsonCapacity.occupied; n < l; n++){
			data = json[n].data;
			// change only if node is not marked
			if(json[n] != markedNode){
				data.$fillColor = nodeFillColor["nodeFamily"];
				data.$color = nodeStrokeColor["nodeFamily"];
			}
		}
		// change port colors
		for(var n = jsonCapacity.max+1, l = json.length; n < l; n++){
			data = json[n].data;
			data.$color = nodeStrokeColor[data.$type];
			if(data.$type == "crossBox"){
				data.$fillColor = nodeFillColor["nodeFamily"];
			}
			// change edge colors
			if(edColor){
				adja = data.adjacencies;
				for(a = 0, al = adja.length; a < al; a++){
					adja[a].data.$color = edgeColor;
				}
			}
		}
	}
	
	/**
	Permits or prohibits node and edge deletion/creation and hides the deletion-cross
	accordingly.
	@param deleteCreate - if true, one can change the graph in any way
	@param move - if true, one can change the graph in any way
	*/
	this.setReadOnly = function(deleteCreate, move){
				
		if(deleteCreate != readOnly.deleteCreate){
			// set crossBoxes (in)visible
			var data;
			for(var n=jsonCapacity.max+1, l= json.length; n < l; n++){
				data = json[n].data;
				if(data.$type == "crossBox"){
					data.$visible = deleteCreate;
					
				}
			}
			readOnly.deleteCreate = deleteCreate;
		}
		
		if(move != undefined){
			readOnly.move = move;
		}
	}
		
	/**
	 Adds a Node to the graph.
	 @param xPosition - the horizontal position of the added node. 0 is the center of the canvas' starting position
	 @param xPosition - the vertical position of the added node. 0 is the center of the canvas' starting position
	 @param nodeFamily - describes some properties of the node: (id, name, nodeClass, tooltip)
	 @param repositoryPorts - an array of properties for the input ports (id, name, tooltip)
	 @param inputPorts - an array of properties for the input ports (id, name, tooltip)
	 @param outputPorts - an array of properties for the output ports (id, name, tooltip)
	 @param nodeType - a string-identifier for a certain group of nodefamilies eg. Filter, Repository
	 @param forced - if true, no event listener will be called and the node will be created no matter what
	 */
	this.addNode = function(xPosition, yPosition, nodeFamily, repositoryPorts, inputPorts, outputPorts, nodeType, forced){
	
	// determine array size
	var countRepo = 0,
		countInput = 0,
		countOutput = 0,
		iconSpace = 0;
		
	if(repositoryPorts){
		countRepo = repositoryPorts.length;
		// check if it is an array
		if(!validArg(caller, repositoryPorts, 'object') || !countRepo){
			countRepo = 0;
		}
	}
	if(inputPorts){
		countInput = inputPorts.length;
		// check if it is an array
		if(!validArg(caller, inputPorts, 'object') || !countInput){
			countInput = 0;
		}
	}
	if(outputPorts){
		countOutput = outputPorts.length;
		// check if it is an array
		if(!validArg(caller, outputPorts, 'object') || !countOutput){
			countOutput = 0;
		}
	}
	
	// check for other illegal arguments
	var caller = 'addNode()';
	if(!validArg(caller, xPosition, 'number')){
		xPosition = 0;
	}
	if(!validArg(caller, yPosition, 'number')){
		yPosition = 0;
	}
	if(!validArg(caller, nodeType, 'string')){
		nodeType = 'Filter';
	}
	if(!validArgNode(caller, nodeFamily)){
		return;
	}
	
	// check if we have enough space reserved
	var max = jsonCapacity.max;
	var occ = jsonCapacity.occupied;
	if(occ == max){
		var families = json.slice(0, max);
		var ports = json.slice(max);
		
		for(var n = 0; n < max; n++){
			families.push(null);
		}
		max *= 2;
		jsonCapacity.max = max;
		json = families.concat(ports);
	}
	
	var maxPorts = Math.max(countInput, countRepo+countOutput+0.5),
		size = vertexLabelSize*2;
		
	// insert some space between repo ports and output ports, only if both are present
	if(maxPorts > countInput && (countRepo == 0 || countOutput == 0)){
		maxPorts-= 0.5;
	}
	
	// check if this nodeType has a defined icon
	if(nodeIcons[nodeType] != undefined){
		iconSpace = 6;
	}
		
	var width = (nodeFamily.name.length+4+iconSpace)*vertexLabelSize,
		height = size + 
			Math.max( 2*size, size/2 + maxPorts * size);
	
	// the classname may be longer than the diplayed name
	if( nodeFamily.nodeClass != undefined){
		width = Math.max(width,  (nodeFamily.nodeClass.length+6+iconSpace)*(vertexLabelSize-2));
	}
	
	// add big node box
	// change node color, depending on its type
	var nodeColor = nodeFillColor["nodeFamily"],
		strokeColor = nodeStrokeColor["nodeFamily"];
	var newNode = {
		  "data": {
			"$dim": size,					
			"$type": "nodeFamily",
			"$jsonIndex": occ,
			"$portCount": countRepo+countInput+countOutput,
			"$portIndex": json.length - max,
			"$icon": nodeIcons[nodeType],
			"$width": width,
			"$height": height,
			"$color": strokeColor,
			"$fillColor": nodeColor,
			"$nodeClass": nodeFamily.nodeClass,
			"$nodeType": nodeType,
			"$xPos": xPosition,
			"$yPos": yPosition,
			"$tooltip": nodeFamily.tooltip
		  }, 
		  "id": nodeFamily.id, 
		  "name": nodeFamily.name
	};
	json[occ] = newNode;
	
	// add closeButton
		var closeButton ={
			  "adjacencies": [], 
			  "data": {
				"$dim": size,					
				"$type": "crossBox",
				"$family": newNode,
				"$xPos": xPosition+ width/2,
				"$yPos": yPosition- height/2,
				"$relativeX": width/2,
				"$relativeY": -height/2,
				"$color": strokeColor,
				"$fillColor": nodeColor,
				"$visible": readOnly.deleteCreate
			   }, 
			  "id": nodeFamily.id+".close", 
			  "name": "x"
		};
	json.push(closeButton);
	json[occ].data.$portCount++;
	
	// Loop Info: 
	// This loop adds all port types to the temporary port array.
	// For fast position computation, the relative position
	// of the first port to the nodeFamily-box is computed for each port type
	// it is stored in relativeX, relativeY
	
	for(var portType = 0; portType < 3; portType++){
		var x, y, relativeX, relativeY,
			loopPorts, loopType;
		switch(portType){
			case 0: // input ports
				relativeX = -width/2;
				relativeY = (0.75 - countInput/2 )*size;
				loopPorts = inputPorts;
				loopType = "inputPort";
				break;
		
			case 1: // repository ports
				var repoSpace = 0.5;
				if(countOutput == 0){
					repoSpace = 0;
				}
				relativeX = width/2;
				relativeY = (0.75 - (countOutput+countRepo+repoSpace)/2 )*size;
				loopPorts = repositoryPorts;
				loopType = "repositoryPort";
				break;
				
			case 2: // output ports
				var repoSpace = 0.5;
				if(countRepo == 0){
					repoSpace = 0;
				}
				relativeX = width/2;
				relativeY = (0.75 - (countOutput-countRepo-repoSpace)/2 )*size;
				loopPorts = outputPorts;
				loopType = "outputPort";
				break;
		}
		
		// continue only if ports exist
		if(loopPorts != null && loopPorts.length > 0){
		
			x = xPosition+ relativeX;
			y = yPosition+ relativeY;
			
			for(var p=0; p < loopPorts.length; p++){
				var port = loopPorts[p];
				
				// only attempt to add a node if it has an id
				if(validArgNode(caller, port)){
				
					var newPort ={
						  "adjacencies": [], 
						  "data": {
							"$dim": size,					
							"$type": loopType,
							"$xPos": x,
							"$yPos": y,
							"$color": nodeStrokeColor[loopType],
							"$fillColor": nodeColor,
							"$tooltip": port.tooltip}, 
						  "id": nodeFamily.id+"."+port.id, 
						  "name": port.name};
						  
					if(p == 0){
						newPort.data.$relativeX= relativeX;
						newPort.data.$relativeY= relativeY;
					}
					json.push(newPort);
					y += size;
				}
			}
		}
	}
	
		// call listener
		if(!forced){
			callListener("onCreateNode", [newNode]);
		}
		jsonCapacity.occupied++;
	}

	/**
		Removes a Node and all its attached edges from the graph.
		@param nodeFamily - the node which is to be removed
		@param forced - if true, the node will be removed unconditionally and no
						event listeners will be called
	*/
	this.removeNode =  function(nodeFamily, forced){
	
		// check for valid arguments
		if(!validArgNode("removeNode()", nodeFamily)){
			return;
		}
		// call listener
		if(!forced && !callListener("onRemoveNode", [nodeFamily])){
			return;
		}
		
		// unmark node
		if (markedNode && markedNode.id == nodeFamily.id){
			markedNode = null;
		}
		
		var deleteFrom = nodeFamily.data.$portIndex + jsonCapacity.max, 
			deleteUntil = deleteFrom+ nodeFamily.data.$portCount,
			familyIndex = nodeFamily.data.$jsonIndex,
			occ = --jsonCapacity.occupied; // decrement occupied spaces
		
		// delete nodeFamily and close button
		delete json[familyIndex];
		delete json[deleteFrom];
		
		// delete ports
		for(var n = deleteFrom+1;  n < deleteUntil;  n++){
			var portID = json[n].id;
			
			// remove all incoming edges
			for(var e = 0;  e < json.length;  e++){
				var incNode = json[e];
				if(incNode != undefined){
					removeEdge(incNode.id, portID, true);	
				}
			}
			
			delete json[n];
		}
		
		// remove space that was used for ports
		var deletionSum = deleteUntil-deleteFrom; 
		json.splice(deleteFrom, deletionSum);
		selectedNode = null;
		
		// update nodeFamily indices and remove the deletion gap
		// in the array space by shifting all following families up
		var data;
		for(var n = familyIndex;  n < occ;  n++){
			json[n] = json[n+1];
			data = json[n].data;
			data.$jsonIndex--
			data.$portIndex-= deletionSum;
			
		}
		json[occ] = null;
		
		// check if we should remove some space
		var max = jsonCapacity.max;
		if( (occ*4) < max && max > 10){
			var ports = json.slice(max);
			
			max /= 2;
			jsonCapacity.max = max;
			var families = json.slice(0, max);
			json = families.concat(ports);
		}
		return;
	 }
	 
	 /**
		Prepares a drag operation for a node, caching all graph nodes that need to be moved.
		This speeds up the moveNode() function considerably.
		@param node - the node which will be moved
	*/
	 function prepareNodeMove(node){
		
		var nodes = [];
		var familyIndex = node.data.$jsonIndex,
			moveFrom = node.data.$portIndex+ jsonCapacity.max, 
			moveUntil = moveFrom+ node.data.$portCount;
		
		// memorize the nodeFamily
		nodes.push(node);
		
		// memorize all ports and the crossbox
		var subnode;
		for(var i= moveFrom; i< moveUntil; i++){
			subnode = fd.graph.getNode(json[i].id);
			nodes.push(subnode);
			
		}
		
		navi.draggedNodes = nodes;
	 }
	 
	 /**
		Moves a nodeFamily and its subnodes to the specified position.
		@param xPos - the horizontal position of the moved node. 0 is the center of the canvas' starting position
		@param yPos - the vertical position of the moved node. 0 is the center of the canvas' starting position
	 */
	 function moveNode(xPos, yPos){
		var nodes = navi.draggedNodes,
			node = nodes[0],
			dim = node.data.$dim;
		var x = xPos,
			y = yPos;
			
		// snap to grid
		if(grid.snap){
			var gridDim = grid.data.$dim,
				left = (xPos - node.data.$width/2),
				top = (yPos - node.data.$height/2),
				right = (xPos + node.data.$width/2),
				bottom = (yPos + node.data.$height/2),
				
				offLeft = Math.abs(left % gridDim),
				offTop = Math.abs(top % gridDim),
				offRight = Math.abs(right % gridDim),
				offBottom = Math.abs(bottom % gridDim),
				threshold = gridDim/6;
		
			if(top > 0){
				offTop = gridDim - offTop;
			}
			if(bottom > 0){
				offBottom = gridDim - offBottom;
			}
			if(left > 0){
				offLeft = gridDim - offLeft;
			}
			if(right > 0){
				offRight = gridDim - offRight;
			}
			
			// snap to closest gridline
			if(offLeft < threshold) {
				x += offLeft;
			}
			if(offRight < threshold) {
				x += offRight;
			}
			else if(offLeft > gridDim - threshold){
				x -= gridDim-offLeft;
			}
			else if(offRight > gridDim - threshold){
				x -= gridDim-offRight;
			}
			
			if(offTop < threshold) {
				y += offTop;
			}
			
			else if(offTop > gridDim - threshold){
				y -= gridDim-offTop;
			}
			
			else if(offBottom < threshold) {
				y += offBottom;
			}
			
			else if(offBottom > gridDim - threshold){
				y -= gridDim-offBottom;
			}
		}
	 
	 
		// set position of nodeFamily
		node.pos.setc(x, y);
		
		// set position of ports and crossbox
		var nodeX, nodeY;
		for(var i= 1, l = nodes.length; i < l; i++){
			node = nodes[i];
			
			if(node.data.$relativeX){
				nodeX = x+ node.data.$relativeX;
				nodeY = y+ node.data.$relativeY;
			}
			node.pos.setc(nodeX,nodeY);
			nodeY += dim;
		}
	 }
	 
	 /**
		Saves the position of a node, so that it will be placed there again,
		when the graph is refreshed. This function will only be called once
		when a node drag operation has finished.
	 */
	 function saveNodeMove(){
		var node = navi.draggedNodes[0],
			familyIndex = node.data.$jsonIndex,
			moveFrom = node.data.$portIndex+ jsonCapacity.max-1, 
			nodes = navi.draggedNodes,
			pos = node.pos.getc(true);
		
		// set pos of family box
		var data= json[familyIndex].data;
		data.$xPos = pos.x;
		data.$yPos = pos.y;
		
		// set pos of ports
		for(var i= 1, l = nodes.length; i < l; i++){
			data = json[moveFrom+i].data;
			pos = nodes[i].pos.getc(true);
			
			data.$xPos = pos.x;
			data.$yPos = pos.y;
		}
		navi.draggedNodes = null;
	 }

  
  /**
   *  Adds a directed edge between two nodes to the graph.
   *  @param sourceID - the id of the node from which the edge starts
   *  @param targetID - the id of the node where the edge ends
   *  @param edgeLabel - the label of the edge. Can be null if no label is required
   *  @param forced - if true, creates the edge unconditionally and does not trigger 
					  a listener event
   *  @return true - if the edge was successfully added
   */
	this.addEdge = function(sourceID, targetID, edgeLabel, forced){
		
		// check for valid arguments
		if(!validArg("addEdge()", sourceID, "string")){
			return false;
		}
		if(!validArg("addEdge()", targetID, "string")){
			return false;
		}
		if(edgeLabel && !validArg("addEdge()", edgeLabel, "string")){
			edgeLabel = null;
		}
		
		// look up the source node and the nodeFamilys of both nodes
		var source, sourceFamily, targetFamily, target;
		
		var sourceFamilyID = sourceID.substring(0, sourceID.indexOf(".")), 
			targetFamilyID = targetID.substring(0, targetID.indexOf(".")),
			loopNode;
		// look for nodefamilies
		for(var n = 0, l = jsonCapacity.occupied; n < l; n++){
	
			loopNode = json[n];
			
			if(loopNode.id == sourceFamilyID){
				sourceFamily = loopNode
			}
			if(loopNode.id == targetFamilyID){
				targetFamily = loopNode
			}
		} 
		
		// look for ports
		for(var n = jsonCapacity.max, l = json.length; n < l; n++){
			loopNode = json[n];
			
			if(loopNode.id == sourceID){
				source = loopNode;
			}
			if(loopNode.id == targetID){
				target = loopNode;
			}
		}
		// call listener if the edge is not connected to the dummy node and check for permission
		if(!forced && sourceID != mouseNode.id && targetID != mouseNode.id){
			if (!callListener("onCreateEdge", [sourceFamily, targetFamily, source, target])){
				return false;
			}
		}
		
		// if edge is connected to mouse, there is no targetFamily...
		var targetFamilyID = null;
		if(targetFamily != undefined){
			targetFamilyID = targetFamily.id;
		}
		
		var adja = source.adjacencies;
		// source node does not exist?
		if(adja == undefined){
			return false;
		}
	
		var edge ={
				  "nodeTo": targetID, 
				  "nodeFrom": sourceID,
				  "data": {"$direction" : [ sourceID, targetID ],
						   "$targetFamily" : targetFamilyID,
						   "$color" : edgeColor},
				  };
		
		if(edgeLabel){
			edge.data.$label = edgeLabel;
        }
		
		adja.push(edge);
		
		if(sourceID == mouseNode.id || targetID == mouseNode.id){
			edge.data.$type = "mouseArrow";
		}
		
		return true;
	}
  
  /**
	Removes the edge between two nodes.
	@param sourceID - the if of the source Node
	@param targetID - the id of the target Node
	@param forced - if true, removes the edge unconditionally and
					does not trigger a listener event
	*/
  this.removeEdge = function(sourceID, targetID, forced){
	// check for valid arguments
	if(!validArg("removeEdge()", sourceID, "string")){
		return;
	}
	if(!validArg("removeEdge()", targetID, "string")){
		return;
	}
  
	// look up the source node and the nodeFamilys of both nodes
	var source, sourceFamily, targetFamily, target;
	
	var sourceFamilyID = sourceID.substring(0, sourceID.indexOf(".")), 
		targetFamilyID = targetID.substring(0, targetID.indexOf(".")),
		loopNode;
		
	// look for ports
	for(var n = jsonCapacity.max, l = json.length; n < l; n++){
		loopNode = json[n];
		if(loopNode != undefined){
			if(loopNode.id == sourceID){
				source = loopNode;
			}
			if(loopNode.id == targetID){
				target = loopNode;
			}
		}
	}
		
	if(!forced){
		// look for nodefamilies
		for(var n = 0, l = jsonCapacity.occupied; n < l; n++){

			loopNode = json[n];
			
			if(loopNode.id == sourceFamilyID){
				sourceFamily = loopNode
			}
			if(loopNode.id == targetFamilyID){
				targetFamily = loopNode
			}
		} 
	
		// call listener and ask for permission to delete
		if(sourceID != mouseNode.id && targetID != mouseNode.id){
			if(!callListener("onRemoveEdge", [sourceFamily, targetFamily, source, target])){
				return;
			}
		}
	}
	
	// nothing to delete?
	if(source == undefined){
		return;
	}
	var adja = source.adjacencies;
	
	for(var a=0; a< adja.length; a++){
		if(adja[a].nodeTo == targetID){
			delete adja[a];
			adja.splice(a,1);
			break;
		}
	}
	
  }
  
  /**
  Returns a node from the json array
  @param nodeID - the id of the desired node
  @return - the node object or null if the node does not exist
  */
  this.getNode = function(nodeID){
	// check for valid arguments
	if(!validArg("getNode()", nodeID, "string")){
		return;
	}
	for(var n=0, l = json.length; n < l; n++){
		var node = json[n];
		if(node && node.id == nodeID){
			return json[n];
		}
	}
	return null;
  }
  
  
  
  /**
	Highlights exactly one node or edge, and un-highlights the previously highlighted
	node or edge.
	@param node - the node or edge object which will be highlighted
	*/
  function setHighlight(node){
		// animation parameters
		var trans = $jit.Trans.Quint.easeOut, 
			dur= 200;
		// clean up the old highlight
		if(hover != null){
			
			if(hover.nodeFrom != undefined){
				hover.setData('lineWidth', 1, 'end');
				hover.setData('color', edgeColor, 'end');
			}
			else{
				var type = hover.data.$type;
				if(markedNode != null && hover.id == markedNode.id){
					hover.setData('color', markedNode.data.$color, 'end');
				}
				else{
					hover.setData('color', nodeStrokeColor[type], 'end');
				}
				
			}
			hover = null;
		}
		
		// do we highlight something new?
		if(node != null){
			if(node.nodeFrom != undefined){
				node.setData('color', edgeColorFocus, 'end');
				node.setData('lineWidth', 2, 'end');
			}else{
				node.setData('color', nodeColorFocus, 'end');
			}
			hover = node;
		}else{
			// slower fade away transition
			trans = $jit.Trans.Quint.easeIn;
			dur = 400;
		}
		
		// Animations overwrite each other. If we let this animation
		// execute while we delete an edge, the onComplete() function
		// of the edge animation will not be called
		fd.animate({  
				modes: ['edge-property:lineWidth:color',
						'node-property:color'],
				transition: trans,  
				duration: dur
		}); 
  }
  
  /**
	Highlights exactly one node, and un-highlights the previously highlighted
	node.
	@param node - the node or edge object which will be highlighted.
				  Can be null if no new node shall be highlighted
	@param strokeColor - the color of the node stroke or a port's fillcolor.
						 Can be null for no change.
	@param fillColor - the fill color of a nodefamily
						Can be null for no change.
	*/
  this.markNode = function(node, strokeColor, fillColor){
  
		// check for valid arguments
		if(node && !validArgNode("markNode()", node)){
			return;
		}
		if(strokeColor && !validArgColor("markNode()", strokeColor)){
			return;
		}
		if(fillColor && !validArgColor("markNode()", fillColor)){
			return;
		}
		
		// clean up the old marking
		if(markedNode != null){
			
			if(markedNode.nodeFrom != undefined){
				// ignore edges
			}
			else{
				var type = markedNode.data.$type;
				markedNode.data.$color = nodeStrokeColor[type];
				
				// change FillColor only if it exists
				if(nodeFillColor[type]){
					markedNode.data.$fillColor = nodeFillColor[type];
				}
			}
			markedNode = null;
		}
		
		// do we highlight something new?
		if(node && node != null){
			// convert dom node to json node
		
			if(node.nodeFrom != undefined){
				// ignore edges
			}else{
				var jsonNode = getNode(node.id);
				if(!jsonNode){
					callListener("onError", ["Error in function markNode(): Marked Node was deleted."]);
					return;
				}
				
				if(strokeColor != null){
					jsonNode.data.$color = strokeColor;
				}
				
				// change FillColor only if it exists
				if(fillColor != null  && nodeFillColor[node.data.$type]){
					jsonNode.data.$fillColor = fillColor;
				}
			}
			markedNode = jsonNode;
		}
  }
    
  /**
	Initializes the Graph, enabling canvas navigation and tooltips.
	Also the json array is initialized with a dummy mouse node and
	what happens on various mouse events is determined.
	*/
  this.initGraph = function(){
	json = [ null, null, null, null, null, null, null, null, null, null, 
			{
			  "adjacencies": [], 
			  "data": {
				"$dim": 0,
				"$type": "none",
				"&xPos": 0,
				"&typeSelected" : null,
				"&yPos": 0,
			  }, 
			  "id": "#DUMMY_MOUSE_NODE", 
			  "name": "",
			  "alpha": 0
		}];
	  
	  fd = new $jit.FlowGraph({
		//id of the visualization container
		injectInto: visContainer,
		//Enable zooming and panning
		//with scrolling and DnD
		Navigation: {
		  enable: true,
		  //Enable panning events only if we're dragging the empty
		  //canvas (and not a node).
		  panning: 'avoid nodes',
		  zooming: 40 //zoom speed. higher is more sensible
		},
		// Change node and edge styles such as
		// color and width.
		// These properties are also set per node
		// with dollar prefixed data-properties in the
		// JSON structure.
		Node: {
		  overridable: true,
		  CanvasStyles: {  
			  lineWidth: 2  
			}  
		  
		},
		Edge: {
		  overridable: true,
		  color: edgeColor,
		  lineWidth: 1,
		  dim: vertexLabelSize,
		  type: 'flowArrow'
		},
		//Native canvas text styling  
		Label: {  
			type: 'HTML', //Native or HTML  
			size: 10,  
			style: 'bold'  
		}, 
		//Add Tips  
		  Tips: {  
			enable: true,  
			type: 'Native',
			onShow: function(tip, node) {  
			  var message = node.data.$tooltip;
			  
			  // do not show a tooltip if it is undefined
			  if(message == undefined){
				tip.style.visibility = "hidden";
			  }
			  else{
				tip.innerHTML = "<div class=\"tip-text\">" + message + "</div>";  
				tip.style.visibility = "visible";
			  }
			}  
		 },  
		// Add node events
		Events: {
		  enable: true,
		  enableForEdges: true,
		  type: 'Native',
		  
		  /**
		   *	EVENT:	OnRightClick
		   */
		  onRightClick: function(node, eventInfo, e) {
			callListener("onRightClick", [node, eventInfo, e]);
		  },
		  
		  /**
		   *	EVENT:	OnDragCancel
		   */
		  onDragCancel: function(node, eventInfo, e) {
			callListener("onDragCancel", [node, eventInfo, e]);
		  },
		  
		  /**
		   *	EVENT:	OnMouseEnter
		   */
		  onMouseEnter: function(node, eventInfo, e) {
			if(node.id == mouseNode.id){
				return;
			}
			
			// ignore some highlights if we are dragging an edge
			if(selectedNode != null && 
				(node.data.$type != "inputPort" 
				&& node.data.$type != "outputPort" 
				&& node.data.$type != "repositoryPort")){
				return;
			}
			
			// check if object is an edge or a node
			mouseOverNode = (node.nodeFrom == undefined);
			mouseOverEdge = !mouseOverNode;
			
			setHighlight(node);
			if(readOnly.move && node.data.$type == "nodeFamily"){
				fd.canvas.getElement().style.cursor = 'move';
			}else if (readOnly.deleteCreate){
				fd.canvas.getElement().style.cursor = 'pointer';
			}
			callListener("onMouseEnter", [node, eventInfo, e]);
		  },
		  
		  /**
		   *	EVENT:	OnMouseMove
		   */
		  onMouseMove: function(node, eventInfo, e) {
		  /*if(!node && navi.isDragging){
			var mousePos = eventInfo.getPos(),
				deltaX = mousePos.x - navi.dragX,
				deltaY = mousePos.y - navi.dragY;
			fd.canvas.translate(deltaX, deltaY);
			deltaX = mousePos.x;
			deltaY = mousePos.y;
		  }
		  */
			// are we dragging an edge with the mouse ?
			if(selectedNode != null){
				var pos = eventInfo.getPos();
				mouseNode.pos.setc(pos.x, pos.y-2);
				fd.plot();
			}
			callListener("onMouseMove", [node, eventInfo, e]);
		  },
		  
		  /**
		   *	EVENT:	OnMouseLeave
		   */
		  onMouseLeave: function(node, eventInfo, e) {
			fd.canvas.getElement().style.cursor = '';
			mouseOverNode = false;
			mouseOverEdge = false;
			
			setHighlight(null);
			callListener("onMouseLeave", [node, eventInfo, e]);
		  },
		  
		  /**
		   *	EVENT:	OnDragStart
		   */
		  onDragStart: function(node, eventInfo, e) {
		  var mousePos = eventInfo.getPos();
		  
			if(node == false || node.id == mouseNode.id ||
			   node.data.$type != "nodeFamily"){
			    navi.dragX = e.pageX;
				navi.dragY = e.pageY;
				navi.isDragging = true;
				return;
			}
			
			// memorize where we drag the node instead on centering
			// it to the mouse pointer
			var	nodePos = node.pos.getc(true);
				
			navi.dragX = nodePos.x - mousePos.x;
			navi.dragY = nodePos.y - mousePos.y;
			
			// we memorize all subnodes of the dragged nodefamily, to speed up
			// dragging
			prepareNodeMove(node);
			
			callListener("onDragStart", [node, eventInfo, e]);
		  },
		  
		  /**
		   *	EVENT:	OnDragMove
		   */
		  onDragMove: function(node, eventInfo, e) {
			if(!readOnly.move || !node || 
				node.id == mouseNode.id || node.data.$type != "nodeFamily"){
					return;
			}
			var pos = eventInfo.getPos();
			moveNode(pos.x+navi.dragX,
					 pos.y+navi.dragY);
			callListener("onDragMove", [node, eventInfo, e]);
			fd.plot();
			
		  },
		  
		  /**
		   *	EVENT:	OnDragEnd
		   */
		  onDragEnd: function(node, eventInfo, e) {
			if(node.data.$type == "nodeFamily"){
				saveNodeMove();
			}
			fd.plot();
			callListener("onDragEnd", [node, eventInfo, e]);
		  },
		  
		  /**
		   *	EVENT:	OnTouchMove
		   */
		  onTouchMove: function(node, eventInfo, e) {
			$jit.util.event.stop(e); //stop default touchmove event
			this.onDragMove(node, eventInfo, e);
		  },
		  
		  /**
		   *	EVENT:	OnClick
		   */
		  onClick: function(node, eventInfo, e){
		  
		  if(node == false){
			var deltaX = e.pageX - navi.dragX,
				deltaY = e.pageY - navi.dragY;
			navi.centerX -= deltaX;
			navi.centerY -= deltaY;
			//fd.canvas.translate(deltaX, deltaY);
		  }
		  else if(mouseOverNode && readOnly.deleteCreate){
		  
			switch(node.data.$type){
				case "crossBox":
					removeNode(node.data.$family);
					refresh();
					break;
					
				case "nodeFamily":
					break;
					
				case "outputPort":
				case "inputPort":
				case "repositoryPort":
					// no selection yet
					if(selectedNode == null){
						selectedNode = {"id": node.id, "from": node.data.$type};
						
						// add edge to mouseNode
						if(selectedNode.from == "inputPort"){
							addEdge(mouseNode.id, node.id);
						}
						else{
							addEdge(node.id, mouseNode.id);
						}
						var pos = node.pos.getc(true);
						refresh();
						
						// set mouse position
						mouseNode.pos.setc(pos.x, pos.y);
						fd.plot();
						return;
					}
					// click outside the edge drag box
					else{
						// add Edge if the selectedNode differs from the clickedNode
						if(selectedNode.from == "inputPort"){
							var label = selectedNode.label;
							addEdge(node.id, selectedNode.id, label);
						}
						else{
							var label = selectedNode.label;
							addEdge(selectedNode.id, node.id, label);
						}
					}
					break;
			}
			
		  }
		  else if(mouseOverEdge && readOnly.deleteCreate){
			// no selection yet
				if(selectedNode == null){
					selectedNode = {"id":node.data.$direction[0], "from":"outputPort", "label":node.data.$label};//nodeTo
					// remove selectedEdge
					var label = node.data.$label;
					removeEdge(selectedNode.id, node.data.$direction[1]);
					// add edge to mouseNode
					addEdge(selectedNode.id, mouseNode.id, label);
					var pos = eventInfo.getPos();
					refresh();
					// set mouse position
					mouseNode.pos.setc(pos.x, pos.y-2);
					fd.plot();
					return;
				}
		  }
		  
		  // remove selection and mouse-edge upon clicking anywhere
		  if(selectedNode != null){
		  
			if(selectedNode.from == "inputPort"){
				removeEdge(mouseNode.id, selectedNode.id, true);
			}else{
				removeEdge(selectedNode.id, mouseNode.id, true);
			}
			
			selectedNode = null;
			refresh();
		  }
		  callListener("onClick", [node, eventInfo, e]);
		  navi.isDragging = false;
		  }
		},
		
		
		//Number of iterations for the FD algorithm
		iterations: 0,
		//Edge length
		levelDistance: 0,
	
	  });
	  
	  
	  // load JSON data.
	  fd.loadGraphFlowJSON(json,jsonCapacity.max);
	  // compute positions
	  refresh();
	  // end
	}
	
	// initialize graph
	initGraph();
	return this;
}